@@ -1,19 +1,17 @@
 //! Implement thread-local storage.
 
-use std::collections::BTreeMap;
 use std::collections::btree_map::Entry as BTreeEntry;
 use std::collections::hash_map::Entry as HashMapEntry;
+use std::collections::BTreeMap;
 
 use log::trace;
 
 use rustc_data_structures::fx::FxHashMap;
 use rustc_middle::ty;
-use rustc_target::abi::{Size, HasDataLayout};
+use rustc_target::abi::{HasDataLayout, Size};
+use rustc_target::spec::abi::Abi;
 
-use crate::{
-    HelpersEvalContextExt, InterpResult, MPlaceTy, Scalar, StackPopCleanup, Tag, ThreadId,
-    ThreadsEvalContextExt,
-};
+use crate::*;
 
 pub type TlsKey = u128;
 
@@ -65,10 +63,14 @@ impl<'tcx> Default for TlsData<'tcx> {
 impl<'tcx> TlsData<'tcx> {
     /// Generate a new TLS key with the given destructor.
     /// `max_size` determines the integer size the key has to fit in.
-    pub fn create_tls_key(&mut self, dtor: Option<ty::Instance<'tcx>>, max_size: Size) -> InterpResult<'tcx, TlsKey> {
+    pub fn create_tls_key(
+        &mut self,
+        dtor: Option<ty::Instance<'tcx>>,
+        max_size: Size,
+    ) -> InterpResult<'tcx, TlsKey> {
         let new_key = self.next_key;
         self.next_key += 1;
-        self.keys.insert(new_key, TlsEntry { data: Default::default(), dtor }).unwrap_none();
+        self.keys.try_insert(new_key, TlsEntry { data: Default::default(), dtor }).unwrap();
         trace!("New TLS key allocated: {} with dtor {:?}", new_key, dtor);
 
         if max_size.bits() < 128 && new_key >= (1u128 << max_size.bits() as u128) {
@@ -107,7 +109,7 @@ impl<'tcx> TlsData<'tcx> {
         &mut self,
         key: TlsKey,
         thread_id: ThreadId,
-        new_data: Option<Scalar<Tag>>
+        new_data: Option<Scalar<Tag>>,
     ) -> InterpResult<'tcx> {
         match self.keys.get_mut(&key) {
             Some(TlsEntry { data, .. }) => {
@@ -140,14 +142,18 @@ impl<'tcx> TlsData<'tcx> {
         &mut self,
         thread: ThreadId,
         dtor: ty::Instance<'tcx>,
-        data: Scalar<Tag>
+        data: Scalar<Tag>,
     ) -> InterpResult<'tcx> {
         if self.dtors_running.contains_key(&thread) {
             // UB, according to libstd docs.
-            throw_ub_format!("setting thread's local storage destructor while destructors are already running");
+            throw_ub_format!(
+                "setting thread's local storage destructor while destructors are already running"
+            );
         }
         if self.macos_thread_dtors.insert(thread, (dtor, data)).is_some() {
-            throw_unsup_format!("setting more than one thread local storage destructor for the same thread is not supported");
+            throw_unsup_format!(
+                "setting more than one thread local storage destructor for the same thread is not supported"
+            );
         }
         Ok(())
     }
@@ -176,16 +182,14 @@ impl<'tcx> TlsData<'tcx> {
         key: Option<TlsKey>,
         thread_id: ThreadId,
     ) -> Option<(ty::Instance<'tcx>, Scalar<Tag>, TlsKey)> {
-        use std::collections::Bound::*;
+        use std::ops::Bound::*;
 
         let thread_local = &mut self.keys;
         let start = match key {
             Some(key) => Excluded(key),
             None => Unbounded,
         };
-        for (&key, TlsEntry { data, dtor }) in
-            thread_local.range_mut((start, Unbounded))
-        {
+        for (&key, TlsEntry { data, dtor }) in thread_local.range_mut((start, Unbounded)) {
             match data.entry(thread_id) {
                 BTreeEntry::Occupied(entry) => {
                     if let Some(dtor) = dtor {
@@ -233,22 +237,29 @@ trait EvalContextPrivExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
     fn schedule_windows_tls_dtors(&mut self) -> InterpResult<'tcx> {
         let this = self.eval_context_mut();
         let active_thread = this.get_active_thread();
-        assert_eq!(this.get_total_thread_count(), 1, "concurrency on Windows not supported");
+        assert_eq!(this.get_total_thread_count(), 1, "concurrency on Windows is not supported");
         // Windows has a special magic linker section that is run on certain events.
         // Instead of searching for that section and supporting arbitrary hooks in there
         // (that would be basically https://github.com/rust-lang/miri/issues/450),
         // we specifically look up the static in libstd that we know is placed
         // in that section.
-        let thread_callback = this.eval_path_scalar(&["std", "sys", "windows", "thread_local", "p_thread_callback"])?;
-        let thread_callback = this.memory.get_fn(thread_callback.not_undef()?)?.as_instance()?;
+        let thread_callback = this.eval_path_scalar(&[
+            "std",
+            "sys",
+            "windows",
+            "thread_local_key",
+            "p_thread_callback",
+        ])?;
+        let thread_callback = this.memory.get_fn(thread_callback.check_init()?)?.as_instance()?;
 
         // The signature of this function is `unsafe extern "system" fn(h: c::LPVOID, dwReason: c::DWORD, pv: c::LPVOID)`.
         let reason = this.eval_path_scalar(&["std", "sys", "windows", "c", "DLL_THREAD_DETACH"])?;
         let ret_place = MPlaceTy::dangling(this.machine.layouts.unit, this).into();
         this.call_function(
             thread_callback,
+            Abi::System { unwind: false },
             &[Scalar::null_ptr(this).into(), reason.into(), Scalar::null_ptr(this).into()],
-            Some(ret_place),
+            Some(&ret_place),
             StackPopCleanup::None { cleanup: true },
         )?;
 
@@ -269,8 +280,9 @@ trait EvalContextPrivExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
             let ret_place = MPlaceTy::dangling(this.machine.layouts.unit, this).into();
             this.call_function(
                 instance,
+                Abi::C { unwind: false },
                 &[data.into()],
-                Some(ret_place),
+                Some(&ret_place),
                 StackPopCleanup::None { cleanup: true },
             )?;
 
@@ -297,20 +309,20 @@ trait EvalContextPrivExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
         let dtor = match this.machine.tls.fetch_tls_dtor(last_key, active_thread) {
             dtor @ Some(_) => dtor,
             // We ran each dtor once, start over from the beginning.
-            None => {
-                this.machine.tls.fetch_tls_dtor(None, active_thread)
-            }
+            None => this.machine.tls.fetch_tls_dtor(None, active_thread),
         };
         if let Some((instance, ptr, key)) = dtor {
-            this.machine.tls.dtors_running.get_mut(&active_thread).unwrap().last_dtor_key = Some(key);
+            this.machine.tls.dtors_running.get_mut(&active_thread).unwrap().last_dtor_key =
+                Some(key);
             trace!("Running TLS dtor {:?} on {:?} at {:?}", instance, ptr, active_thread);
             assert!(!this.is_null(ptr).unwrap(), "data can't be NULL when dtor is called!");
 
             let ret_place = MPlaceTy::dangling(this.machine.layouts.unit, this).into();
             this.call_function(
                 instance,
+                Abi::C { unwind: false },
                 &[ptr.into()],
-                Some(ret_place),
+                Some(&ret_place),
                 StackPopCleanup::None { cleanup: true },
             )?;
 
@@ -325,15 +337,11 @@ trait EvalContextPrivExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
 
 impl<'mir, 'tcx: 'mir> EvalContextExt<'mir, 'tcx> for crate::MiriEvalContext<'mir, 'tcx> {}
 pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx> {
-
     /// Schedule an active thread's TLS destructor to run on the active thread.
     /// Note that this function does not run the destructors itself, it just
     /// schedules them one by one each time it is called and reenables the
     /// thread so that it can be executed normally by the main execution loop.
     ///
-    /// FIXME: we do not support yet deallocation of thread local statics.
-    /// Issue: https://github.com/rust-lang/miri/issues/1369
-    ///
     /// Note: we consistently run TLS destructors for all threads, including the
     /// main thread. However, it is not clear that we should run the TLS
     /// destructors for the main thread. See issue:
@@ -346,30 +354,34 @@ pub trait EvalContextExt<'mir, 'tcx: 'mir>: crate::MiriEvalContextExt<'mir, 'tcx
             // This is the first time we got asked to schedule a destructor. The
             // Windows schedule destructor function must be called exactly once,
             // this is why it is in this block.
-            if this.tcx.sess.target.target.target_os == "windows" {
+            if this.tcx.sess.target.os == "windows" {
                 // On Windows, we signal that the thread quit by starting the
                 // relevant function, reenabling the thread, and going back to
                 // the scheduler.
                 this.schedule_windows_tls_dtors()?;
-                return Ok(())
+                return Ok(());
             }
         }
+        // The remaining dtors make some progress each time around the scheduler loop,
+        // until they return `false` to indicate that they are done.
+
         // The macOS thread wide destructor runs "before any TLS slots get
         // freed", so do that first.
         if this.schedule_macos_tls_dtor()? {
             // We have scheduled a MacOS dtor to run on the thread. Execute it
             // to completion and come back here. Scheduling a destructor
             // destroys it, so we will not enter this branch again.
-            return Ok(())
+            return Ok(());
         }
         if this.schedule_next_pthread_tls_dtor()? {
             // We have scheduled a pthread destructor and removed it from the
             // destructors list. Run it to completion and come back here.
-            return Ok(())
+            return Ok(());
         }
 
         // All dtors done!
         this.machine.tls.delete_all_thread_tls(active_thread);
+        this.thread_terminated()?;
 
         Ok(())
     }
