@@ -1,7 +1,10 @@
-// ignore-windows: Unwind panicking does not currently work on Windows
-// normalize-stderr-test "[^ ]*libcore/(macros|mem)/mod.rs[0-9:]*" -> "$$LOC"
+// normalize-stderr-test "[^ ]*core/[a-z_/]+.rs[0-9:]*" -> "$$LOC"
+// normalize-stderr-test "catch_panic\.rs:[0-9]{2}" -> "catch_panic.rs:LL"
+// We test the `align_offset` panic below, make sure we test the interpreter impl and not the "real" one.
+// compile-flags: -Zmiri-symbolic-alignment-check
 #![feature(never_type)]
-#![allow(unconditional_panic)]
+#![allow(unconditional_panic, non_fmt_panic)]
+
 use std::panic::{catch_unwind, AssertUnwindSafe};
 use std::cell::Cell;
 
@@ -75,7 +78,6 @@ fn main() {
     // Assertion and debug assertion
     test(None, |_old_val| { assert!(false); loop {} });
     test(None, |_old_val| { debug_assert!(false); loop {} });
-    test(None, |_old_val| { unsafe { (1 as *const i32).read() }; loop {} }); // trigger debug-assertion in libstd
 
     eprintln!("Success!"); // Make sure we get this in stderr
 }
@@ -99,7 +101,7 @@ fn test(expect_msg: Option<&str>, do_panic: impl FnOnce(usize) -> !) {
         eprintln!("Caught panic message (&str): {}", s);
         Some(*s)
     } else {
-        eprintln!("Failed get caught panic message.");
+        eprintln!("Failed to get caught panic message.");
         None
     };
     if let Some(expect_msg) = expect_msg {
