@@ -9,7 +9,7 @@ extern crate rustc_span;
 
 #[cfg(feature = "ipc")]
 use rustc_driver::Compilation;
-use rustc_driver::{run_compiler, Callbacks};
+use rustc_driver::{Callbacks, RunCompiler};
 use rustc_interface::interface;
 #[cfg(feature = "ipc")]
 use rustc_interface::Queries;
@@ -34,12 +34,9 @@ pub fn run() -> Result<(), ()> {
     #[cfg(feature = "ipc")]
     let (mut shim_calls, file_loader) = match std::env::var("RLS_IPC_ENDPOINT").ok() {
         Some(endpoint) => {
-            #[allow(deprecated)] // Windows doesn't work with lazily-bound reactors
-            let reactor = rt.reactor().clone();
-            let connection =
-                ipc::connect(endpoint, &reactor).expect("Couldn't connect to IPC endpoint");
-            let client: ipc::Client =
-                rt.block_on(connection).expect("Couldn't connect to IPC endpoint");
+            let client: ipc::Client = rt
+                .block_on(async { ipc::connect(endpoint).await })
+                .expect("Couldn't connect to IPC endpoint");
             let (file_loader, callbacks) = client.split();
 
             (
@@ -75,9 +72,13 @@ pub fn run() -> Result<(), ()> {
     };
 
     rustc_driver::install_ice_hook();
-    rustc_driver::catch_fatal_errors(|| run_compiler(&args, &mut shim_calls, file_loader, None))
-        .map(|_| ())
-        .map_err(|_| ())
+    rustc_driver::catch_fatal_errors(move || {
+        let mut compiler = RunCompiler::new(&args, &mut shim_calls);
+        compiler.set_file_loader(file_loader);
+        compiler.run()
+    })
+    .map(|_| ())
+    .map_err(|_| ())
 }
 
 #[derive(Default)]
@@ -109,7 +110,6 @@ impl Callbacks for ShimCalls {
     ) -> Compilation {
         use rustc_session::config::Input;
 
-        use futures::future::Future;
         use rls_ipc::rpc::{Crate, Edition};
         use std::collections::{HashMap, HashSet};
 
@@ -121,7 +121,7 @@ impl Callbacks for ShimCalls {
         let sess = compiler.session();
         let input = compiler.input();
 
-        let cwd = &sess.working_dir.0;
+        let cwd = &sess.working_dir.local_path_if_available();
 
         let src_path = match input {
             Input::File(ref name) => Some(name.to_path_buf()),
@@ -136,6 +136,7 @@ impl Callbacks for ShimCalls {
             edition: match sess.edition() {
                 rustc_span::edition::Edition::Edition2015 => Edition::Edition2015,
                 rustc_span::edition::Edition::Edition2018 => Edition::Edition2018,
+                rustc_span::edition::Edition::Edition2021 => Edition::Edition2021,
             },
         };
 
@@ -144,7 +145,7 @@ impl Callbacks for ShimCalls {
             input_files.entry(file).or_default().insert(krate.clone());
         }
 
-        if let Err(e) = callbacks.input_files(input_files).wait() {
+        if let Err(e) = futures::executor::block_on(callbacks.input_files(input_files)) {
             log::error!("Can't send input files as part of a compilation callback: {:?}", e);
         }
 
@@ -157,8 +158,6 @@ impl Callbacks for ShimCalls {
         compiler: &interface::Compiler,
         queries: &'tcx Queries<'tcx>,
     ) -> Compilation {
-        use futures::future::Future;
-
         let callbacks = match self.callbacks.as_ref() {
             Some(callbacks) => callbacks,
             None => return Compilation::Continue,
@@ -169,9 +168,6 @@ impl Callbacks for ShimCalls {
         let input = compiler.input();
         let crate_name = queries.crate_name().unwrap().peek().clone();
 
-        // Guaranteed to not be dropped yet in the pipeline thanks to the
-        // `config.opts.debugging_opts.save_analysis` value being set to `true`.
-        let expanded_crate = &queries.expansion().unwrap().peek().0;
         queries.global_ctxt().unwrap().peek_mut().enter(|tcx| {
             // There are two ways to move the data from rustc to the RLS, either
             // directly or by serialising and deserialising. We only want to do
@@ -179,7 +175,6 @@ impl Callbacks for ShimCalls {
 
             // This version passes via JSON, it is more easily backwards compatible.
             // save::process_crate(state.tcx.unwrap(),
-            //                     state.expanded_crate.unwrap(),
             //                     state.analysis.unwrap(),
             //                     state.crate_name.unwrap(),
             //                     state.input,
@@ -189,14 +184,15 @@ impl Callbacks for ShimCalls {
             // This version passes directly, it is more efficient.
             rustc_save_analysis::process_crate(
                 tcx,
-                &expanded_crate,
                 &crate_name,
                 &input,
                 None,
                 CallbackHandler {
                     callback: &mut |a| {
                         let analysis = unsafe { ::std::mem::transmute(a.clone()) };
-                        if let Err(e) = callbacks.complete_analysis(analysis).wait() {
+                        if let Err(e) =
+                            futures::executor::block_on(callbacks.complete_analysis(analysis))
+                        {
                             log::error!(
                                 "Can't send analysis as part of a compilation callback: {:?}",
                                 e
@@ -213,14 +209,14 @@ impl Callbacks for ShimCalls {
 
 #[cfg(feature = "ipc")]
 fn fetch_input_files(sess: &rustc_session::Session) -> Vec<PathBuf> {
-    let cwd = &sess.working_dir.0;
+    let cwd = &sess.working_dir.local_path_if_available();
 
     sess.source_map()
         .files()
         .iter()
         .filter(|fmap| fmap.is_real_file())
         .filter(|fmap| !fmap.is_imported())
-        .map(|fmap| fmap.name.to_string())
+        .map(|fmap| fmap.name.prefer_local().to_string())
         .map(|fmap| src_path(Some(cwd), fmap).unwrap())
         .collect()
 }
