@@ -1,6 +1,7 @@
-use env_logger;
-use failure::{err_msg, format_err, Error as FailureError, Fail};
+use anyhow::{format_err, Result};
+
 use io::Error as IoError;
+use thiserror::Error;
 
 use rustfmt_nightly as rustfmt;
 
@@ -59,20 +60,28 @@ enum Operation {
 }
 
 /// Rustfmt operations errors.
-#[derive(Fail, Debug)]
+#[derive(Error, Debug)]
 pub enum OperationError {
     /// An unknown help topic was requested.
-    #[fail(display = "Unknown help topic: `{}`.", _0)]
+    #[error("Unknown help topic: `{0}`.")]
     UnknownHelpTopic(String),
     /// An unknown print-config option was requested.
-    #[fail(display = "Unknown print-config option: `{}`.", _0)]
+    #[error("Unknown print-config option: `{0}`.")]
     UnknownPrintConfigTopic(String),
     /// Attempt to generate a minimal config from standard input.
-    #[fail(display = "The `--print-config=minimal` option doesn't work with standard input.")]
+    #[error("The `--print-config=minimal` option doesn't work with standard input.")]
     MinimalPathWithStdin,
     /// An io error during reading or writing.
-    #[fail(display = "io error: {}", _0)]
+    #[error("{0}")]
     IoError(IoError),
+    /// Attempt to use --check with stdin, which isn't currently
+    /// supported.
+    #[error("The `--check` option is not supported with standard input.")]
+    CheckWithStdin,
+    /// Attempt to use --emit=json with stdin, which isn't currently
+    /// supported.
+    #[error("Using `--emit` other than stdout is not supported with standard input.")]
+    EmitWithStdin,
 }
 
 impl From<IoError> for OperationError {
@@ -184,7 +193,7 @@ fn is_nightly() -> bool {
 }
 
 // Returned i32 is an exit code
-fn execute(opts: &Options) -> Result<i32, FailureError> {
+fn execute(opts: &Options) -> Result<i32> {
     let matches = opts.parse(env::args().skip(1))?;
     let options = GetOptsOptions::from_matches(&matches)?;
 
@@ -206,7 +215,7 @@ fn execute(opts: &Options) -> Result<i32, FailureError> {
             Ok(0)
         }
         Operation::ConfigOutputDefault { path } => {
-            let toml = Config::default().all_options().to_toml().map_err(err_msg)?;
+            let toml = Config::default().all_options().to_toml()?;
             if let Some(path) = path {
                 let mut file = File::create(path)?;
                 file.write_all(toml.as_bytes())?;
@@ -224,8 +233,8 @@ fn execute(opts: &Options) -> Result<i32, FailureError> {
             let file = PathBuf::from(path);
             let file = file.canonicalize().unwrap_or(file);
 
-            let (config, _) = load_config(Some(file.parent().unwrap()), Some(options.clone()))?;
-            let toml = config.all_options().to_toml().map_err(err_msg)?;
+            let (config, _) = load_config(Some(file.parent().unwrap()), Some(options))?;
+            let toml = config.all_options().to_toml()?;
             io::stdout().write_all(toml.as_bytes())?;
 
             Ok(0)
@@ -238,10 +247,18 @@ fn execute(opts: &Options) -> Result<i32, FailureError> {
     }
 }
 
-fn format_string(input: String, options: GetOptsOptions) -> Result<i32, FailureError> {
+fn format_string(input: String, options: GetOptsOptions) -> Result<i32> {
     // try to read config from local directory
     let (mut config, _) = load_config(Some(Path::new(".")), Some(options.clone()))?;
 
+    if options.check {
+        return Err(OperationError::CheckWithStdin.into());
+    }
+    if let Some(emit_mode) = options.emit_mode {
+        if emit_mode != EmitMode::Stdout {
+            return Err(OperationError::EmitWithStdin.into());
+        }
+    }
     // emit mode is always Stdout for Stdin.
     config.set().emit_mode(EmitMode::Stdout);
     config.set().verbose(Verbosity::Quiet);
@@ -271,7 +288,7 @@ fn format(
     files: Vec<PathBuf>,
     minimal_config_path: Option<String>,
     options: &GetOptsOptions,
-) -> Result<i32, FailureError> {
+) -> Result<i32> {
     options.verify_file_lines(&files);
     let (config, config_path) = load_config(None, Some(options.clone()))?;
 
@@ -319,7 +336,7 @@ fn format(
     // that were used during formatting as TOML.
     if let Some(path) = minimal_config_path {
         let mut file = File::create(path)?;
-        let toml = session.config.used_options().to_toml().map_err(err_msg)?;
+        let toml = session.config.used_options().to_toml()?;
         file.write_all(toml.as_bytes())?;
     }
 
@@ -486,7 +503,7 @@ struct GetOptsOptions {
     verbose: bool,
     config_path: Option<PathBuf>,
     inline_config: HashMap<String, String>,
-    emit_mode: EmitMode,
+    emit_mode: Option<EmitMode>,
     backup: bool,
     check: bool,
     edition: Option<Edition>,
@@ -498,7 +515,7 @@ struct GetOptsOptions {
 }
 
 impl GetOptsOptions {
-    pub fn from_matches(matches: &Matches) -> Result<GetOptsOptions, FailureError> {
+    pub fn from_matches(matches: &Matches) -> Result<GetOptsOptions> {
         let mut options = GetOptsOptions::default();
         options.verbose = matches.opt_present("verbose");
         options.quiet = matches.opt_present("quiet");
@@ -519,7 +536,7 @@ impl GetOptsOptions {
                     options.error_on_unformatted = Some(true);
                 }
                 if let Some(ref file_lines) = matches.opt_str("file-lines") {
-                    options.file_lines = file_lines.parse().map_err(err_msg)?;
+                    options.file_lines = file_lines.parse()?;
                 }
             } else {
                 let mut unstable_options = vec![];
@@ -548,7 +565,7 @@ impl GetOptsOptions {
         options.inline_config = matches
             .opt_strs("config")
             .iter()
-            .flat_map(|config| config.split(","))
+            .flat_map(|config| config.split(','))
             .map(
                 |key_val| match key_val.char_indices().find(|(_, ch)| *ch == '=') {
                     Some((middle, _)) => {
@@ -574,7 +591,7 @@ impl GetOptsOptions {
                 return Err(format_err!("Invalid to use `--emit` and `--check`"));
             }
 
-            options.emit_mode = emit_mode_from_emit_str(emit_str)?;
+            options.emit_mode = Some(emit_mode_from_emit_str(emit_str)?);
         }
 
         if let Some(ref edition_str) = matches.opt_str("edition") {
@@ -590,11 +607,13 @@ impl GetOptsOptions {
         }
 
         if !rust_nightly {
-            if !STABLE_EMIT_MODES.contains(&options.emit_mode) {
-                return Err(format_err!(
-                    "Invalid value for `--emit` - using an unstable \
-                     value without `--unstable-features`",
-                ));
+            if let Some(ref emit_mode) = options.emit_mode {
+                if !STABLE_EMIT_MODES.contains(emit_mode) {
+                    return Err(format_err!(
+                        "Invalid value for `--emit` - using an unstable \
+                         value without `--unstable-features`",
+                    ));
+                }
             }
         }
 
@@ -643,8 +662,8 @@ impl CliOptions for GetOptsOptions {
         }
         if self.check {
             config.set().emit_mode(EmitMode::Diff);
-        } else {
-            config.set().emit_mode(self.emit_mode);
+        } else if let Some(emit_mode) = self.emit_mode {
+            config.set().emit_mode(emit_mode);
         }
         if self.backup {
             config.set().make_backup(true);
@@ -662,11 +681,11 @@ impl CliOptions for GetOptsOptions {
     }
 
     fn config_path(&self) -> Option<&Path> {
-        self.config_path.as_ref().map(|p| &**p)
+        self.config_path.as_deref()
     }
 }
 
-fn edition_from_edition_str(edition_str: &str) -> Result<Edition, FailureError> {
+fn edition_from_edition_str(edition_str: &str) -> Result<Edition> {
     match edition_str {
         "2015" => Ok(Edition::Edition2015),
         "2018" => Ok(Edition::Edition2018),
@@ -674,7 +693,7 @@ fn edition_from_edition_str(edition_str: &str) -> Result<Edition, FailureError>
     }
 }
 
-fn emit_mode_from_emit_str(emit_str: &str) -> Result<EmitMode, FailureError> {
+fn emit_mode_from_emit_str(emit_str: &str) -> Result<EmitMode> {
     match emit_str {
         "files" => Ok(EmitMode::Files),
         "stdout" => Ok(EmitMode::Stdout),
